local name, ns = ...

local class = select(2, UnitClass("player"))
if class ~= "MONK" and class ~= "WARLOCK" then return end

local DB
local char_key = UnitName("player").."-"..GetRealmName()
local curMapWidth, curMapHeight = math.huge, math.huge
local curMap = GetMapInfo()
local curFloor = GetCurrentMapDungeonLevel()
local Portal = {}
local distance_format
local pX, pY

local atan2 = math.atan2
local UnitHasVehicleUI = UnitHasVehicleUI
local IsInPetBattle = C_PetBattles.IsInBattle
local GetCurrentMapZone = GetCurrentMapZone
local abs = math.abs
local str_fmt = string.format

local defaults = {
	MONK = {
		texpath_body = [[Interface\AddOns\Teleportie\Media\Textures\Transcendence1.blp]],
		texpath_direction = [[Interface\AddOns\Teleportie\Media\Textures\Direction1.blp]],
		texpath_bg = [[Interface\AddOns\Teleportie\Media\Textures\Background1.blp]],
		texColor_far = {255/255, 15/255, 0/255, .5}, --red
		texColor_near = {83/255, 255/255, 197/255, 1}, --green
		textColor_far = {255/255, 15/255, 0/255}, --red
		textColor_near = {20/255, 255/255, 80/255}, --green	
		decimals = 1,
		insets = 0,
		frame_size = 48,
		font_height = 12,
		Portal = {},
		updatePeriodMapOpened = 0.5,
	},
	WARLOCK = {
	}
}
defaults.WARLOCK = defaults.MONK

local function UpdateOldVersionSettings()
	for k, v in pairs(defaults[class]) do
		if DB[k] == nil then
			DB[k] = v
		end
	end
end

local f = CreateFrame("FRAME", name, UIParent)
f:Hide()
f:SetSize(48, 48)
f:SetPoint("CENTER")
f.Tex_Icon = f:CreateTexture(nil, "BORDER")
f.Tex_Icon:SetAllPoints()
f.Tex_Circle = f:CreateTexture(nil, "BACKGROUND")
f.Tex_Circle:SetAllPoints()
f.FS_Distance = f:CreateFontString(nil, "BORDER")
f.FS_Distance:SetFont([[Fonts\FRIZQT__.TTF]], 12, "OUTLINE")
f.FS_Distance:SetNonSpaceWrap(true)
f.FS_Distance:SetJustifyH("CENTER")
f.FS_Distance:SetPoint("BOTTOM", f, "BOTTOM", 0, 12)
f.Direction = f:CreateTexture(nil, "ARTWORK")
f.Direction:SetPoint("CENTER")
f.Direction:SetAllPoints()
f.Dir_Ag = f.Direction:CreateAnimationGroup()
f.Dir_Rot = f.Dir_Ag:CreateAnimation("Rotation")
f.Dir_Rot:SetDuration(0)
f.Dir_Rot:SetEndDelay(10)
f.Dir_Ag:SetLooping("REPEAT")
f.Dir_Ag:Play()
f:SetClampedToScreen(true)
f:EnableMouse(true)
f:SetMovable(true)
f:RegisterForDrag("RightButton")
f:SetFrameStrata("HIGH")
f:RegisterUnitEvent("UNIT_SPELLCAST_SUCCEEDED", "player")
f:RegisterEvent("PLAYER_LOGIN")
f:RegisterEvent("PLAYER_ALIVE")
f:RegisterEvent("PLAYER_ENTERING_WORLD")
f:RegisterEvent("ZONE_CHANGED_NEW_AREA")

local T_mapOpened, T_expireCheck = f:CreateAnimationGroup(), f:CreateAnimationGroup(), f:CreateAnimationGroup() --animation timers
local t_mO = T_mapOpened:CreateAnimation()
T_mapOpened:SetLooping("REPEAT")

local t_eC = T_expireCheck:CreateAnimation()
local function ExpireCheck(check_in, handler)
	t_eC:SetDuration(check_in)
	t_eC:SetScript("OnFinished", handler)
	T_expireCheck:Play()
end


function f:UpdateAppearance(s)
	f.Tex_Icon:SetTexture(s.texpath_body)
	f.Tex_Circle:SetTexture(s.texpath_bg)
	f.Direction:SetTexture(s.texpath_direction)
	local insets = s.insets
	f:SetClampRectInsets(-1 * insets, insets, insets, -1 * insets)
	f.FS_Distance:SetTextHeight(s.font_height)
	distance_format = "%00000."..s.decimals.."f"
	f:SetSize(s.frame_size, s.frame_size)
	if s.framePos then
		self:SetPoint(unpack(s.framePos))
	else
		self:SetPoint("CENTER", UIParent, "CENTER")
	end
	f:SetSize(s.frame_size, s.frame_size)
	t_mO:SetDuration(s.updatePeriodMapOpened)
end

local mapOpened, mapId, level = false

local function MarkPosition(savedX, savedY, savedInstanceId, returnFlag) --call with parameters to restore saved info or without to mark new one
	local instance = select(8, GetInstanceInfo())
	if savedInstanceId and savedInstanceId ~= instance then wipe(Portal) return end
	if mapOpened then
		mapId = GetCurrentMapAreaID()
		level = GetCurrentMapDungeonLevel()
		SetMapToCurrentZone()
	end
	Portal.instance = instance
	local x, y = GetPlayerMapPosition("player")
	local _, x1, y1, x2, y2 = GetCurrentMapDungeonLevel()
	if not x1 then
		x1, y1, x2, y2 = select(2, GetCurrentMapZone())
	end
	if x1 < x2 then
        local temp = x1
        x1 = x2
        x2 = temp
    end
    if y1 < y2 then
        local temp = y1
        y1 = y2
        y2 = temp
    end
    curMapWidth, curMapHeight = abs(x1-x2), abs(y1-y2)
	local RealX, RealY = savedX or x1 - abs(x1-x2)*x, savedY or y1 - abs(y1-y2)*y
	pX, pY = abs(RealX - x1), abs(RealY - y1)
	Portal.RealX, Portal.RealY = RealX, RealY

	if mapOpened and not returnFlag then
		SetMapByID(mapId)
		SetDungeonMapLevel(level)
	end
	f:Show()
end
local function MarkPositionAndTime()
	Portal.h, Portal.m = GetGameTime()
	MarkPosition()
end

f:SetScript("OnEnter", function(self)

end)
f:SetScript("OnLeave", function(self)
	
end)
f:SetScript("OnDragStart", function(self, ...)
	if IsShiftKeyDown() then
		self:StartMoving()
	end
end)
f:SetScript("OnDragStop", function(self, ...)
	self:StopMovingOrSizing()
	DB.framePos = {self:GetPoint()}
end)
f:SetScript("OnEvent", function(self, event, ...) 
	if event == "UNIT_SPELLCAST_SUCCEEDED" then
		local spellId = select(5, ...)
		if spellId == 101643 or spellId == 48018 then
			MarkPositionAndTime()
		elseif spellId == 119996 then
			MarkPosition()
		end
	elseif event == "PLAYER_LOGIN" then
		Teleportie_DB = Teleportie_DB or {}
		Teleportie_DB[char_key] = Teleportie_DB[char_key] or defaults[class]
		DB = Teleportie_DB[char_key]
		UpdateOldVersionSettings()
		f:UpdateAppearance(DB)
		Portal = DB.Portal
	elseif (event == "PLAYER_ENTERING_WORLD" or event == "ZONE_CHANGED_NEW_AREA") and Portal.RealX then
		MarkPosition(Portal.RealX, Portal.RealY, Portal.instance)
	elseif event == "PLAYER_ALIVE" then
		self:Hide()
	end
end)
f:SetScript("OnMouseWheel", function(self, delta)
	local size = self:GetSize() + (0.5 * delta)
	self:SetSize(size, size)
	DB.frame_size = size
end)

local function Update_wMapClosed() 	
	local x, y = GetPlayerMapPosition("player")
	local x_diff, y_diff = pX - curMapWidth*x, pY - curMapHeight*y
	local dist = sqrt(x_diff ^ 2 + y_diff ^ 2)
	local facing = GetPlayerFacing()
	local direction = atan2(x_diff, y_diff)
	f.FS_Distance:SetText(str_fmt(distance_format, dist))
	f.Dir_Rot:SetRadians(direction - facing + PI)
	if dist > 100 and class == "MONK" then
		f:Hide()
		wipe(Portal)
	elseif dist < 39.99 then
		f.Tex_Icon:SetVertexColor(unpack(DB.texColor_near))
		f.FS_Distance:SetTextColor(unpack(DB.textColor_near))
	else
		f.Tex_Icon:SetVertexColor(unpack(DB.texColor_far))
		f.FS_Distance:SetTextColor(unpack(DB.textColor_far))
	end
end
f:SetScript("OnUpdate", Update_wMapClosed)

local function Update_wMapOpened() 
	local x, y = GetPlayerMapPosition("player")
	if x == 0 then
		mapId, level = GetCurrentMapAreaID(), GetCurrentMapDungeonLevel()
		SetMapToCurrentZone()
	else
		mapId = nil
	end

	if mapId == -1 then return end

	local _, x1, y1, x2, y2 = GetCurrentMapDungeonLevel()
	if not x1 then
		x1, y1, x2, y2 = select(2, GetCurrentMapZone())
	end
	if not x1 then return end
	if x1 < x2 then
        local temp = x1
        x1 = x2
        x2 = temp
    end
    if y1 < y2 then
        local temp = y1
        y1 = y2
        y2 = temp
    end

	local x, y = GetPlayerMapPosition("player")
	local x_diff, y_diff = (x1 - abs(x1-x2)*x) - Portal.RealX, (y1 - abs(y1-y2)*y) - Portal.RealY
	local dist = sqrt(x_diff ^ 2 + y_diff ^ 2)
	local facing = GetPlayerFacing()
	local direction = atan2(x_diff, y_diff)
	f.FS_Distance:SetText(str_fmt(distance_format, dist))
	f.Dir_Rot:SetRadians(direction - facing + PI)
	if dist > 100 and class == "MONK" then
		f:Hide()
		wipe(Portal)
	elseif dist < 39.99 then
		f.Tex_Icon:SetVertexColor(unpack(DB.texColor_near))
		f.FS_Distance:SetTextColor(unpack(DB.textColor_near))
	else
		f.Tex_Icon:SetVertexColor(unpack(DB.texColor_far))
		f.FS_Distance:SetTextColor(unpack(DB.textColor_far))
	end

	if mapId then
		SetMapByID(mapId)
		SetDungeonMapLevel(level)
	end
end
t_mO:SetScript("OnFinished", Update_wMapOpened)

WorldMapFrame:HookScript("OnShow", function() 
	mapOpened = true
	f:SetScript("OnUpdate", nil)
	T_mapOpened:Play() 
end)
WorldMapFrame:HookScript("OnHide", function() 
	if Portal.RealX then 
		MarkPosition(Portal.RealX, Portal.RealY, Portal.instance, true)
	end 
	mapOpened = false
	T_mapOpened:Stop()
	f:SetScript("OnUpdate", Update_wMapClosed) 
end)